import cv2
import mediapipe as mp




class HandDetector():
    def __init__(self):
        self.mp_hands = mp.solutions.hands
        self.hands = self.mp_hands.Hands(static_image_mode=False,
                               max_num_hands=2,
                               min_tracking_confidence=0.4,
                               min_detection_confidence=0.4)
        self.draw = mp.solutions.drawing_utils


    def process_frame(self, img):

        height, width, _ = img.shape
        results = self.hands.process(img)

        handness_str = ''
        index_finger_tip_str = ''
        if results.multi_hand_landmarks:
            for h_idx, hand in enumerate(results.multi_hand_landmarks):
                self.draw.draw_landmarks(img, hand, self.mp_hands.HAND_CONNECTIONS)
                # get 21 key points of hand
                hand_info = results.multi_hand_landmarks[h_idx]

                # log left and right hands info
                temp_handness = results.multi_handedness[h_idx].classification[0].label
                handness_str += "{}:{} ".format(h_idx, temp_handness)

                cz0 = hand_info.landmark[0].z
                for idx, keypoint in enumerate(hand_info.landmark):
                    cx = int(keypoint.x * width)
                    cy = int(keypoint.y * height)
                    cz = keypoint.z
                    depth_z = cz0 - cz

                    # radius about depth_z
                    radius = int(6 * (1 + depth_z))

                    if idx == 0:
                        img = cv2.circle(img, (cx, cy), radius * 2, (0, 0, 255), -1)
                    elif idx == 8:
                        img = cv2.circle(img, (cx, cy), radius * 2, (193, 182, 255), -1)
                        index_finger_tip_str += '{}:{:.2f} '.format(h_idx, depth_z)
                    elif idx in [1, 5, 9, 13, 17]:
                        img = cv2.circle(img, (cx, cy), radius, (16, 144, 247), -1)
                    elif idx in [2, 6, 10, 14, 18]:
                        img = cv2.circle(img, (cx, cy), radius, (1, 240, 255), -1)
                    elif idx in [3, 7, 11, 15, 19]:
                        img = cv2.circle(img, (cx, cy), radius, (140, 47, 240), -1)
                    elif idx in [4, 12, 16, 20]:
                        img = cv2.circle(img, (cx, cy), radius, (223, 155, 60), -1)
        img = cv2.putText(img, handness_str, (25, 50), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 0), 2)
        img = cv2.putText(img, index_finger_tip_str, (25, 100), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 0, 255), 2)

        return img


if __name__ == '__main__':
    cap = cv2.VideoCapture()
    cap.open(0)
    detector = HandDetector()
    while cap.isOpened():
        success, frame = cap.read()
        if not success:
            raise ValueError("Error")
        frame = cv2.flip(frame, 1)
        frame = detector.process_frame(frame)
        cv2.imshow("hand", frame)
        if cv2.waitKey(1) in [ord('q'), 27]:
            break
    cap.release()
    cv2.destroyAllWindows()